@@ -1,11 +1,13 @@
 import copy
+from types import ModuleType
+from typing import Any, Callable, Dict, List, Optional
 
 from functional import seq
 from jsonschema import validate
 from spacy.matcher import Matcher
 from spacy.tokens import Span
 
-import replacy.custom_patterns as custom_patterns
+from replacy import default_match_hooks
 from replacy.db import get_forms_lookup, get_match_dict, get_match_dict_schema
 from replacy.inflector import Inflector
 from replacy.version import __version__
@@ -25,14 +27,45 @@ expected_properties = (
 
 
 class ReplaceMatcher:
-    def __init__(self, nlp, match_dict=None, forms_lookup=None):
+    """
+    The main unit of functionality. Instantiate with `nlp`, (an instance of spaCy) and a match dict.
+    Usage example, including a module of custom match hooks:
+
+    ```python
+        from replacy import ReplaceMatcher
+        from replacy.db import load_json
+        import spacy
+
+        import my.custom_hooks as ch  # suppose this suggests `excepts=>accepts` under some conditions
+
+
+        nlp = spacy.load("en_core_web_sm")
+        rmatch_dict = load_json("./resources/match_dict.json")
+        rmatcher = ReplaceMatcher(nlp, rmatch_dict, custom_match_hooks=ch)
+        span = rmatcher("She excepts her fate.")[0]
+        span._.suggestions
+        # >>> ['acccepts']
+    ```
+    """
+
+    def __init__(
+        self,
+        nlp,
+        match_dict=None,
+        forms_lookup=None,
+        custom_match_hooks: Optional[ModuleType] = None,
+        allow_multiple_whitespaces=False,
+    ):
+        self.default_match_hooks = default_match_hooks
+        self.custom_match_hooks = custom_match_hooks
         self.nlp = nlp
         self.match_dict = match_dict if match_dict else get_match_dict()
         self.forms_lookup = forms_lookup if forms_lookup else get_forms_lookup()
+        self.allow_multiple_whitespaces = allow_multiple_whitespaces
 
         self.matcher = Matcher(self.nlp.vocab)
         self._init_matcher()
-        self.spans = []
+        self.spans: List[Span] = []
         self.inflector = Inflector(nlp=self.nlp, forms_lookup=self.forms_lookup)
 
         # set custom extensions for any unexpected keys found in the match_dict
@@ -42,7 +75,7 @@ class ReplaceMatcher:
             .distinct()
             .difference(expected_properties)
         )
-        novel_prop_defaults = {}
+        novel_prop_defaults: Dict[str, Any] = {}
         for x in self.match_dict.values():
             for k, v in x.items():
                 if k in novel_properites and k not in novel_prop_defaults.keys():
@@ -72,11 +105,16 @@ class ReplaceMatcher:
         match_dict_schema = get_match_dict_schema()
         validate(instance=match_dict, schema=match_dict_schema)
 
-    def get_predicates(self, match_hooks):
+    def get_predicates(self, match_hooks) -> List[Callable]:
         predicates = []
         for hook in match_hooks:
-            # template - ex. succeeded_by_word
-            template = getattr(custom_patterns, hook["name"])
+            # template - ex. succeeded_by_phrase
+            try:
+                template = getattr(self.default_match_hooks, hook["name"])
+            except AttributeError:
+                # if the hook isn't in custom_match_hooks, this will still
+                # raise an exception. I think that is the correct behavior
+                template = getattr(self.custom_match_hooks, hook["name"])
 
             # predicate - filled template ex. succeeded_by_word("to")
             # will match "in addition to..." but not "in addition, ..."
@@ -92,7 +130,8 @@ class ReplaceMatcher:
             # negate, since positive breaks matching
             # see cb in get_callback
             if bool(hook.get("match_if_predicate_is", False)):
-                pred = getattr(custom_patterns, "neg")(pred)
+                # neg flips the boolean value of a predicate
+                pred = default_match_hooks.neg(pred)
             predicates.append(pred)
         return predicates
 
@@ -170,6 +209,22 @@ class ReplaceMatcher:
         for match_name, ps in self.match_dict.items():
             patterns = copy.deepcopy(ps["patterns"])
 
+            """
+            allow matching tokens separated by multiple whitespaces
+            they may appear after normalizing nonstandard whitespaces
+            ex. "Here␣is␣a\u180E\u200Bproblem." -> "Here␣is␣a␣␣problem."
+            pattern can be preceded and followed by whitespace tokens
+            to keep preceded_by... with and succeeded_by... with match hooks working
+            """
+            if self.allow_multiple_whitespaces:
+
+                white_pattern = {"IS_SPACE": True, "OP": "?"}
+
+                normalized_patterns = [white_pattern]
+                for p in patterns:
+                    normalized_patterns += [p, white_pattern]
+                patterns = normalized_patterns
+
             # remove custom attributes not supported by spaCy Matcher
             for p in patterns:
                 if "TEMPLATE_ID" in p:
@@ -179,12 +234,15 @@ class ReplaceMatcher:
             callback = self.get_callback(match_name, match_hooks)
             self.matcher.add(match_name, callback, patterns)
 
-    def __call__(self, sent: str):
+    def __call__(self, sent):
         # self.spans must be cleared - global
         self.spans = []
-        sent_doc = self.nlp(sent)
+        try:
+            sent.text
+        except AttributeError:
+            sent = self.nlp(sent)
 
         # this fills up self.spans
-        matches = self.matcher(sent_doc)
+        matches = self.matcher(sent)
 
         return self.spans
